require 'rails/generators'

module Interview
  module Generators
    class ViewControlGenerator < ::Rails::Generators::NamedBase
      include Rails::Generators::ResourceHelpers
      
      argument :attributes, type: :array, default: [], banner: "field:type field:type"
      
      source_root File.expand_path("../templates", __FILE__)
      desc "This generator adds View Controls for a model."
      
      def add_erb_templates
        template "list.rb", "app/view_controls/#{file_name}_list.rb"
        template "card.rb", "app/view_controls/#{file_name}_card.rb"
        template "form.rb", "app/view_controls/#{file_name}_form.rb"
      end
      
      protected
      
      def attribute_class_name(attribute)
        return case attribute.type
        when :string then 'StringAttribute'
        when :text then 'TextAttribute'
        when :boolean then 'BooleanAttribute'
        when :date then 'DateAttribute'
        when :option then 'OptionAttribute'
        else 'Attribute'
        end
      end
      
    end
  end
end