module Interview
  class Builder
    
    attr_accessor :html, :curr_parent
    
    def initialize(options={})
      @html = ::Builder::XmlMarkup.new
    end
    
    def render!
      @html.target!
    end
    
    def add!(control, &block)
      if @curr_parent.is_a? MetaControl and not @curr_parent.skip
        @curr_parent.build_child self, control, &block
      else
        control.parent = @curr_parent
        @curr_parent = control
        control.build self, &block
        @curr_parent = control.parent
      end
    end
    
    def add_abstract!(sym, *args, &block)
      if Object.const_defined?(sym.to_s.camelcase) and
         Object.const_get(sym.to_s.camelcase) <= Interview::Control
        control = Object.const_get(sym.to_s.camelcase).new(*args)
      else
        # todo: prüfen, ob Control vorhanden
        control = Object.const_get("Interview::#{sym.to_s.camelcase}").new(*args)
      end
      add! control, &block
    end
    
    def tag!(sym, *args, &block)
      @html.tag!(sym, *args, &block)
    end
    
    def text!(text)
      @html.text!(text)
    end
    
    def <<(text)
      @html << text
    end
    
    def lookup!(attr_name)
      @curr_parent.find_attribute!(attr_name)
    end
    
    def method_missing(sym, *args, &block)
      add_abstract!(sym, *args, &block)
    end
    
  end
end